/**
  * Sampling and More, by Perry R. Cook, July 2015
  * Originally from the Minim FFT demo.
  * but there's no FFTs in this one, just waveforms.
  * For more information about Minim and additional features, 
  * visit http://code.compartmental.net/minim/
  */

import ddf.minim.*;

Minim      minim;
AudioInput in;

boolean capture = false;
boolean sample = false;
boolean qtize = false;
boolean waveform = true;

int BUFFSIZE = 640;
float[] wave = new float[BUFFSIZE];
float[] capt = new float[BUFFSIZE];

int rate = 16;
int quant = 32;
float zoomb = 1.0;

int TEXTX = 450;

float xwoffs,ywoffs,yboffs,xscale,ybscale,ywscale;

void setup()
{
//  size(1920, 1080, P3D);
  size(800, 500, P3D);
  frameRate(15);
  xscale = 1.0*width/BUFFSIZE;
  ywscale = height/5;
  ywoffs = height/5;
  yboffs = 3*height/5;
  ybscale = 2*height/5;
  
  minim = new Minim(this);

  in = minim.getLineIn(Minim.STEREO,BUFFSIZE);
}

void draw()
{
  background(0);
  stroke(255);
  
// Instructions, key controls:
  PFont f2 = createFont("Courier",18,true);
  textFont(f2);
  text("Keys: <sp> = Capture Buffer  s = Show Samples       R/r = SRate Up/Down",10,30);
  text("      +/- = Zoom in/out      / = Show Quantization  Q/q = qtize Up/down =",10,60);
  text(44100/rate,width-360,30);
  text(quant,width-360,60);
// draw the waveforms, draw lissajous, count zero crossings, calculate power
  for(int i = 0; i < BUFFSIZE - 1; i++)   {
    wave[i] = 2.0*in.mix.get(i);
    if (capture) capt[i] = wave[i];
  }
  if (capture) capture = false;
  
  float quantum = 1.0 / quant;
  
  if (sample)  {
      line(rate*xscale*zoomb,yboffs,rate*xscale*zoomb,yboffs+ybscale/2);
      line(2*rate*xscale*zoomb,yboffs,2*rate*xscale*zoomb,yboffs+ybscale/2);
      text("T",1.4*rate*xscale*zoomb,yboffs+ybscale/2-20);
  }
  for(int i = 1; i < BUFFSIZE - 1; i++)   {
    line((i-1)*xscale*zoomb, ywoffs-wave[i-1]*ywscale,i*xscale*zoomb,ywoffs-wave[i]*ywscale);
    if (waveform) {
      line((i-1)*xscale*zoomb, yboffs-capt[i-1]*ybscale,i*xscale*zoomb,yboffs-capt[i]*ybscale);
    }
    if (sample) {
       if (i % rate == 0) {
          line (i*xscale*zoomb,yboffs,i*xscale*zoomb,yboffs-capt[i]*ybscale);
          ellipse(i*xscale*zoomb,yboffs-capt[i]*ybscale,5,5);
       }
    }
    if (qtize)  {
      if (i % rate == 0)  {
//         int step = -quant;
//         while (step*quantum*0.5 < capt[i]) step++;
//         float yval = step*0.5*quantum;
         int yint = int(quant*capt[i]/2);
         float yval = 2.0*(yint+0.5)/quant;
         float xval = i*xscale*zoomb;
         line(xval,yboffs,xval,yboffs-yval*ybscale);
         line(xval,yboffs-yval*ybscale,(i+rate)*xscale*zoomb,yboffs-yval*ybscale);
         xval = (i+rate)*xscale*zoomb;
         line(xval,yboffs,xval,yboffs-yval*ybscale);
      }
    }
  }
  
  stroke(255);
  line(0,yboffs,width,yboffs);
  //  line(0,ywoffs,width,ywoffs);
}

void keyPressed()  {
    if (key == ' ') capture = true;
    if (key == 's') sample = !sample;
    if (key == '/') qtize = !qtize;
    if (key == 'w') waveform = !waveform;
    if (key == 'R') {
      rate /= 2;
      if (rate < 1) rate = 1;
    }
    if (key == 'r') rate *= 2;
    if (key == 'q') {
       quant /= 2;
       if (quant < 1) quant = 1;
    }
    if (key == 'Q') quant *= 2;
    if (key == '+') zoomb *= 2; 
    if (key == '-') {
      zoomb *= 0.5;
      if (zoomb < 1.0) zoomb = 1.0;
    }
}
